/*
 * A recursive data structure (list) of Ratio objects with some
 * useful methods to manage such a list
 *
 * Jim Teresco, The College of Saint Rose, CSC 523, Summer 2014
 *
 * $Id: RatioList.java 3027 2016-10-17 02:58:32Z terescoj $
 */

public class RatioList {

    // our instance variables have room for one Ratio, and
    // then a list of other ratios -- which is exactly
    // the structure we are defining -- so we can define this
    // structure *recursively*
    private Ratio first;
    private RatioList rest;
    
    // we construct a RatioList from a Ratio and another RatioList,
    // which would be null when constructing a brand new RatioList
    // with its first element
    public RatioList(Ratio first, RatioList rest) {
        
        this.first = first;
        this.rest = rest;
    }
    
    // a method to compute the sum of all ratios
    // it's recursive
    public Ratio getSum() {
        
        // if there is no rest, the sum is just first
        if (rest == null) return first;
        
        // otherwise, we have to compute the sum of the rest (using
        // this method) and then add in first, using Ratio's add method
        return first.add(rest.getSum());
    }
    
    // to find the minimum ratio, we also use a recursive method.
    // the minimum is the smaller of first and the minimum of the rest,
    // which we compute using this method recursively
    public Ratio getMin() {
        
        // the smallest out of a single ratio is that ratio
        if (rest == null) return first;
        
        // otherwise, find the min of the rest and compare to first
        Ratio minOfRest = rest.getMin();
        if (minOfRest.getDecimalValue() < first.getDecimalValue()) {
            return minOfRest;
        }
        else {
            return first;
        }
    }
    
    // max is the same idea
    public Ratio getMax() {
        
        // the largest out of a single ratio is that ratio
        if (rest == null) return first;
        
        // otherwise, find the max of the rest and compare to first
        Ratio maxOfRest = rest.getMax();
        if (maxOfRest.getDecimalValue() > first.getDecimalValue()) {
            return maxOfRest;
        }
        else {
            return first;
        }
    }
    
    // reduce all Ratios in the list to lowest terms
    public void reduceAll() {
    
        // reduce the one Ratio we have
        first.reduce();
        
        // and if there are more, reduce recursively
        if (rest != null) rest.reduceAll();
    }
    
    
    // the toString method is recursive!  To print this whole RatioList,
    // we concatenate the result of first's toString with a recursive
    // call to this toString method.  But only do the latter when rest
    // is not null (we are not at the end of the list)
    public String toString() {
        
        String answer = first.toString();
        if (rest != null) {
            answer += ", " + rest.toString();
        }
       
        return answer;
    }
}
