import structure5.*;
/**
 * An iterator class for implementing an iterator over a Vector that
 * returns only the elements at even-numbered indices.
 *
 * Based on VectorIterator from Duane Bailey's stucture package
 *
 * @version $Id: VectorEvenIterator.java 950 2009-10-07 02:09:52Z terescoj $
 * @author, Jim Teresco, 2009
 */
public class VectorEvenIterator<E> extends AbstractIterator<E>
{
    /**
     * The associated vector
     */
    protected Vector<E> theVector;
    /**
     * The index of the current value.
     */
    protected int current;

    /**
     * Construct a vector iterator to traverse vector v
     *
     * @post constructs an initialized iterator associated with v
     * 
     * @param v The underlying vector.
     */
    public VectorEvenIterator(Vector<E> v)
    {
        theVector = v;
        reset();
    }

    /**
     * Reset the vector iterator to the first value in the vector.
     *
     * @post the iterator is reset to the beginning of the traversal
     */
    public void reset()
    {
        current = 0;
    }

    /**
     * Determine if some of the elements have yet to be considered.
     *
     * @post returns true if there is more structure to be traversed
     * 
     * @return True if more elements are to be considered.
     */
    public boolean hasNext()
    {
        return current < theVector.size();
    }

    /**
     * Fetch a reference to the current value.
     *
     * @pre traversal has more elements
     * @post returns the current value referenced by the iterator 
     * 
     * @return A reference to the current value being considered.
     */
    public E get()
    {
        return theVector.get(current);
    }
    
    /**
     * Return current value, and increment iterator to next
     * value to be returned (the next even-numbered entry)
     *
     * @pre traversal has more elements
     * @post increments the iterated traversal by 2
     * 
     * @return A reference to the current value, before increment.
     */
    public E next()
    {
	E result = theVector.get(current);
	current+=2;
        return result;
    }
}

